#version 330

in vec3 fPos;
in vec3 fNormal;

uniform vec3 lightPos;
uniform vec3 viewPos;
uniform vec3 lightColor;
uniform vec4 objectColor;
uniform float shininess;
uniform float specularStrength;

out vec4 fColor;

void main()
{
    // ambient
    float ambientStrength = 0.5;
    vec3 ambient = ambientStrength * lightColor;

    // diffuse
    vec3 norm = normalize(fNormal);
    vec3 lightDir = normalize(lightPos - fPos);
    float diff = max(dot(norm, lightDir), 0.0);
    vec3 diffuse = diff * lightColor;

    // specular
    vec3 specular = vec3(0.0, 0.0, 0.0);
    if (dot(fNormal,lightDir) > 0.0) {
        vec3 viewDir = normalize(viewPos - fPos);
        vec3 reflectDir = reflect(-lightDir, norm);
        float spec = pow(max(dot(viewDir, reflectDir), 0.0), shininess);
        specular = specularStrength * spec * lightColor;
        // specular = clamp(specular, 0.0, 1.0);
    }

    fColor = vec4((ambient + diffuse),1) * objectColor + vec4(specular,1);
}