#pragma once

#include "Scene.h"
#include "Camera.h"

#include <thread>

/**
 * @brief Physics simulation
 * 
 * Simulates the billiard balls and table with a simplified 2D simulation in
 * the xy-plane at z=0. Billiard balls are considered spheres and the table is
 * considered a flat top with 4 infinite collision planes at the table banks.
 * Collisions are solved with positional separation and application of impulses
 * according impulse-based simulation.
 */
class Physics
{
public:
    /** The time that the last frames took to simulate. */
    float dt = 0.f;

    /** Scene object */
    std::shared_ptr<Scene> m_scene;

    /** Whether the physics thread should terminate. */
    bool m_abort = false;

    /** The physics thread that does the simulation integration and collisions. */
    std::shared_ptr<std::thread> m_thread;

    /** Whether balls are currently in motion. */
    bool m_ballsMoving = false;

    void startSimulation( std::shared_ptr<Scene> scene );
    void abortSimulation();

private:
    void asyncTask();
    void collisionBallBall( std::shared_ptr<Body> ball1, std::shared_ptr<Body> ball2 );
    void collisionBallBank( std::shared_ptr<Body> ball, glm::vec3 normal );
};